%% amdf testing
clear;
clc;

%% operation variables to modify
    window_size = 30;    % number of samples to use in amdf for each delay
    shortest_delay = 30; % highest frequency / lowest delay to sample for
    longest_delay = 230; % lowest frequency / highest delay to sample for
    rms_thresh = 5;      % threshold of input signal to allow amdf to output
    sample_offset = 0;   % dc offset of sample
    origin = 1.0;        % psuedo HPS var, the original bin factor 
    harm_fact = 0.20;    % psuedo HPS var, the harmonic proportion to add to origin


%% dependent operation variables from above
    max_num_of_samples_needed = longest_delay + window_size;
    calculated_note(longest_delay) = 0;
    count_thresh = 0;
    amdf_bank = 0;
    delay = 1:longest_delay;
    power_scale = 1/( max_num_of_samples_needed * 1/44100); 
    

%% grab the sample and form "window chunks" from which to calculate the amdf
for figure_num = 1:2
    
    %allow for testing of multiple samples, first sample
    %allows for you to test robustness of algorithm and different
    %frequencies
    if figure_num == 1
        [violin_sample,Fs] = audioread('violin_g3_bowed.aif');
        %[violin_sample,Fs] = audioread('violin_db4_bowed.aif');
        %[violin_sample,Fs] = audioread('violin_a4_bowed.aif');
        %[violin_sample,Fs] = audioread('violin_e5_bowed.aif');
    end
    
    %second sample set
    if figure_num == 2
        %[violin_sample,Fs] = audioread('violin_g3_bowed.aif');
        %[violin_sample,Fs] = audioread('violin_db4_bowed.aif');
        %[violin_sample,Fs] = audioread('violin_a4_bowed.aif');
        [violin_sample,Fs] = audioread('violin_e5_bowed.aif');
    end
    
    %cut up the entire sample into discrete windows of a fixed size
    violin_sample = violin_sample + sample_offset;
    sample_length = 1:length(violin_sample);
    len = length(violin_sample) / max_num_of_samples_needed;
    len = int32(len); %determine integer number of windows for input sample
    rms(len) = 0;
    sample_array = cell(max_num_of_samples_needed,len,1);
    for frame = 1:len
        for sample_num = 1:max_num_of_samples_needed
            sample_cell{sample_num,frame} = violin_sample(sample_num+max_num_of_samples_needed*(frame-1));
        end
    end
    sample_array = cell2mat(sample_cell);
    clear rms;
    count_thresh = 0;


%% operate on windows using amdf, choose highest peak, and accumulate results
for frame_num = 1:len

    % copy current sample into working variable, average, and attain power
    clear sample;
    sample(max_num_of_samples_needed) = 0;
    rms(frame_num) = 0;
    for i = 1:max_num_of_samples_needed
        
        %grab the sample
        sample(i) = sample_array(i,frame_num);
        
        % sum sample for power calculation
        rms(frame_num) = rms(frame_num) + sample(i)^2; 
        
% center clipping seems to destroy the algorithm, so I left it out
%         % center clipping of the input signal
%         if abs(sample(i)) < 0.025
%             sample(i) = 0;
%         end
%         if sample(i) > 0.025
%             sample(i) = sample(i) - 0.025;
%         end
%         if sample(i) < -0.025
%             sample(i) = sample(i) + 0.025;
%         end

    end
    
    % power calculation for amdf result validity
    rms(frame_num) = sqrt(rms(frame_num) * power_scale); 
    
    % formulate amdf result, using specific window size
    clear amdf;
    max_amdf = 0;
    for index = shortest_delay:longest_delay
        amdf(index) = 0;
        for index_samp = 1:window_size
            amdf(index) = amdf(index) + abs( sample(index_samp) - sample(index_samp+index) );
            if amdf(index) > max_amdf
                max_amdf = amdf(index);
            end
        end
        % normalize
        amdf(index) = (1 - amdf(index)/max_amdf); 
        % only allow strong correlations to accumulate
        if amdf(index) < 0.90 
            amdf(index) = 0;
        end
    end

    % hps the amdf result
    for x = shortest_delay:longest_delay
        if x*2 < longest_delay
            amdf(x) = amdf(x)*origin + amdf(x*2+1)*harm_fact + amdf(x*2-1)*harm_fact + amdf(x*2)*harm_fact;
        end
        if x*3 < longest_delay
            amdf(x) = amdf(x)*origin + amdf(x*3+1)*harm_fact + amdf(x*3-1)*harm_fact + amdf(x*3)*harm_fact;
        end
    end
    
    % determine the highest value and store it for statistical analysis
    if rms(frame_num) > rms_thresh
        test_val = 0.0;
        test_index = 0;
        for i = shortest_delay:longest_delay
            if amdf(i) > test_val
                test_val = amdf(i);
                test_index = i;
            end
        end
        if test_index ~= 0
            calculated_note(test_index) = calculated_note(test_index)+1;
            count_thresh = count_thresh + 1;
            amdf_bank = amdf_bank + amdf;
        end
    end
    
end


%% print out the data
    figure(figure_num);
    subplot(4,1,1);
    plot(sample_length,violin_sample);
    title('Input sample captured at 44100hz');

    subplot(4,1,2);
    rms_x = 1:len;
    plot(rms_x,rms,rms_x,rms_thresh);
    title('RMS of audio sample in time with threshold line shown');
    
    max1 = max(amdf_bank);
    amdf_bank = amdf_bank/max1;
    subplot(4,1,3);
    plot(delay,amdf_bank);
    axis([0 275 0 1]);
    title('Accumulated AMDF Spectrum of Input Sample');

    subplot(4,1,4);
    % scale display
    calculated_note = calculated_note/count_thresh; 
    plot(delay, calculated_note);
    axis([0 275 0 1]);
    title('Normalized Percent of Calculated Notes.');
    x = sprintf('%d satisfactory RMS calculations of %d samples', count_thresh, frame);
    xlabel(x);
    [ydat,xdat] = max(calculated_note);
    %add 1 deviation values
    ydat = ydat + calculated_note(xdat-1) + calculated_note(xdat+1); 
    x = sprintf('%d%%  at  %dHz', int16(ydat*100), int16(1/(xdat*44100^-1)));
    text(xdat+5,0.5,x);
end