%% provide a sample file, ensure it is 44100hz sample rate
clear
clc
[violin_sample,Fs] = audioread('violin_e5_bowed.aif');


%% setup some constants
points = 1:1024;
pi = 3.14159265;
mag = 0.54 - 0.46*cos(2*pi*points/(1024-1));
violin_spectrum = 0:43.0664:5512;
calculated_note(41) = 0;
count_thresh = 0;
fft_bank(512) = 0;

%% define the sample sets of the provided file
sample_length = 1:length(violin_sample);
len = length(violin_sample)/1024;
len = int32(len);
sample_array = cell(1024,len,1);
for frame = 1:len
    for sample_num = 1:1024
    sample_cell{sample_num,frame} = violin_sample(sample_num+1024*(frame-1));
    end
end
sample_array = cell2mat(sample_cell);

%% Loop through the samples and determine the notes played during each 1024 sample set
for frame_num = 1:len

    % copy current sample into working variable
    clear sample;
    for i = 1:1024
        sample(i) = sample_array(i,frame_num);
    end
    sample = sample .* mag;

    % take windowed sample and run fft, abs value, look at positive half
    fft_pos = fft(sample);
    fft_pos = fft_pos(1:512);
    fft_pos = abs(fft_pos);
    fft_bank = fft_bank + fft_pos;

    % compute harmonic product spectrum for 172hz to 1720hz
    clear hps;
    for i = 5:41
        % take 3 harmonics
        hps(i) = (fft_pos(i)+fft_pos(i*2)+fft_pos(i*3))/3; 
        % take 5 harmonics
        %hps(i) = (fft_pos(i)+fft_pos(i*2)+fft_pos(i*3)+fft_pos(i*4)+fft_pos(i*5))/5;
        % take 7 harmonics
        %hps(i) = (fft_pos(i)+fft_pos(i*2)+fft_pos(i*3)+fft_pos(i*4)+fft_pos(i*5)+fft_pos(i*6)+fft_pos(i*7))/7;
    end

    % determine the highest value and store it
    test_val = 5.0;
    test_index = 0;
    for i = 5:41
        if hps(i) > test_val
            test_val = hps(i);
            test_index = i;
        end
    end
    if test_index ~= 0
        calculated_note(test_index) = calculated_note(test_index)+1;
        count_thresh = count_thresh + 1;
    end
end

%% print out the data
calculated_note = calculated_note/count_thresh; % scale
subplot(3,1,1);
plot(sample_length,violin_sample);
title('Input sample captured at 44100hz');

max = max(fft_bank);
fft_bank = fft_bank/max;
subplot(3,1,2);
plot(points(1:128)*43,fft_bank(1:128));
title('Normalized Frequency Spectrum of Input Sample');

subplot(3,1,3);
notes_of_interest = 0:43:1720;
plot(notes_of_interest, calculated_note);
title('Normalized Percent of Calculated Notes.');
x = sprintf('%d hits of %d samples', count_thresh, frame);
xlabel(x);